@@ -2,10 +2,13 @@ import time
 
 from tooz import coordination
 
-coordinator = coordination.get_coordinator('memcached://localhost', b'host-1')
+ALIVE_TIME = 5
+
+coordinator = coordination.get_coordinator('zake://', b'host-1')
 coordinator.start()
 
-while True:
+start = time.time()
+while time.time() - start < ALIVE_TIME:
     coordinator.heartbeat()
     time.sleep(0.1)
 
