@@ -90,7 +90,7 @@ class MySQLLock(locking.Lock):
 
     def __del__(self):
         if self.acquired:
-            LOG.warn("unreleased lock %s garbage collected" % self.name)
+            LOG.warning("unreleased lock %s garbage collected", self.name)
 
 
 class MySQLDriver(coordination.CoordinationDriver):
@@ -105,6 +105,7 @@ class MySQLDriver(coordination.CoordinationDriver):
     """
 
     CHARACTERISTICS = (
+        coordination.Characteristics.NON_TIMEOUT_BASED,
         coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
         coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
         coordination.Characteristics.DISTRIBUTED_ACROSS_HOSTS,
